﻿using System;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;
using Infragistics.Documents.Excel;
using Microsoft.Practices.Composite.Presentation.Commands;
using System.Windows.Input;

namespace ExportExcelXg
{
    public class ViewModel
    {
        public ICommand ExportExcel { get; set; }

        public ViewModel()
        {
            ExportExcel = new DelegateCommand<object>(export);
        }

        /// <summary>
        /// Method to answer the user where the file will be save
        /// </summary>
        /// <param name="obj"></param>
        private void export(object obj)
        {
            var dialog = new SaveFileDialog { Filter = "Excel files|*.xls", DefaultExt = "xls" };

            bool? ret = dialog.ShowDialog();
            if (ret != true) return;

            try
            {
                Stream exportStream = dialog.OpenFile();

                //launch the excel export
                Workbook dataWorkbook = this.SaveExcel(obj, "sheetName");
                dataWorkbook.Save(exportStream);
                exportStream.Close();

                MessageBox.Show(
                    "The file " + dialog.SafeFileName + " was successfully generated.");

            }
            catch (IOException)
            {
                MessageBox.Show(
                        "Impossible to acces to file.");
            }
        }

        #region Variables

        private const int InitialColumnsCount = 0;
        private int _currentMergedRow;
        private int _currentMergedColumn;
        private int _countGrouped;
        private int _countNotGrouped;
        private int _outlineLevel = 1;
        private int _outlineLevelNextGroup = 1;
        private int _childrenCount;
        private Worksheet _sheet;
        private IWorksheetCellFormat _iwshcfValide;
        
        #endregion

        /// <summary>
        /// Method to iterate each row of the grid and save it in the excel
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        private Workbook SaveExcel(object obj, string name)
        {
            //get the XamGrid
            var grille = (XamGrid)obj;
            //creation od the workbook
            var workbook = new Workbook(WorkbookFormat.Excel97To2003);

            //sheet name
            _sheet = workbook.Worksheets.Add(name + " - " + DateTime.Now.Day + " " + DateTime.Now.Month + " " + DateTime.Now.Year);

            // Samples to define the style of cells
            //Sample 1
            _iwshcfValide = workbook.CreateNewWorksheetCellFormat();
            _iwshcfValide.Font.ColorInfo = new WorkbookColorInfo(Colors.Green);
            _iwshcfValide.Alignment = HorizontalCellAlignment.Left;

            _currentMergedRow = 0;
            _currentMergedColumn = 0;

            //get the groups, if there's
            var groupByQuery = from row in grille.Rows
                               where row.RowType == RowType.GroupByRow
                               select row;

            //get the simplerows
            var simpleQuery = from row in grille.Rows
                              where row.RowType == RowType.DataRow
                              select row;

            foreach (var query in groupByQuery)
            {
                _childrenCount = 0;
                _countGrouped = 0;
                _countNotGrouped = 0;
                _childrenCount = GetQueryElementItemsCount(query as GroupByRow);
                _outlineLevel = 1;
                _outlineLevelNextGroup = 2;


                //this 'for' create the +/- in front of each group to collapse them
                for (int i = 0; i < _childrenCount; i++)
                    _sheet.Rows[_currentMergedRow + i + 1].OutlineLevel = _outlineLevel;

                _currentMergedColumn = 0;

                //treatment of the rows of the group
                IterateGroupByRow(query as GroupByRow, _outlineLevelNextGroup);
            }

            foreach (var query in simpleQuery)
            {
                //the fisrt row do not scroll with others when ther's no groups
                _sheet.DisplayOptions.PanesAreFrozen = true;
                _sheet.DisplayOptions.FrozenPaneSettings.FrozenRows = 1;

                int currentColumn = 0;
                //creat the header when no groups
                foreach (Column column in grille.Columns)
                {
                    if (column.Visibility == Visibility.Visible)
                    {
                        this.SetCellValue(_sheet.Rows[0].Cells[currentColumn], column.HeaderText);
                        currentColumn++;
                    }
                }
                IterateRow(query);
            }

            return workbook;
        }

        /// <summary>
        /// Permet de connaitre le nombre d'element d'un groupe (ligne + entête)
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        private int GetQueryElementItemsCount(GroupByRow query)
        {
            RowCollection children = query.Rows;
            foreach (Row child in children)
            {
                if (child.RowType == RowType.GroupByRow)
                {
                    _countGrouped++;
                    GetQueryElementItemsCount(child as GroupByRow);
                }
                else
                    _countNotGrouped++;
            }
            return _countGrouped + _countNotGrouped;
        }

        /// <summary>
        /// Method to check each row of a group
        /// </summary>
        /// <param name="groupByRow"></param>
        /// <param name="outlinelevel">le niveau du groupe pour les regroupements dans excel</param>
        private void IterateGroupByRow(GroupByRow groupByRow, int outlinelevel)
        {
            //merge to get an indentation
            WorksheetMergedCellsRegion mergedRegion = _sheet.MergedCellsRegions.Add(_currentMergedRow, _currentMergedColumn, _currentMergedRow, _currentMergedColumn + InitialColumnsCount);
            _sheet.Columns[_currentMergedColumn].Width = 6000;
            if (groupByRow.GroupByData.Value == null)
                mergedRegion.Value = "";
            else
            {
                try
                {
                    DateTime dt = DateTime.ParseExact(groupByRow.GroupByData.Value.ToString(), "dd/MM/yyyy HH:mm:ss", new CultureInfo("fr-FR"));
                    mergedRegion.Value = dt.Date.ToShortDateString();
                }
                catch (FormatException)
                {
                    mergedRegion.Value = groupByRow.GroupByData.Value;
                }
            }

            _currentMergedRow++;
            _currentMergedColumn++;

            if (groupByRow.HasChildren)
            {
                RowCollection groupByRowChildren = groupByRow.Rows;

                //check for each row if it's a group or not
                foreach (Row child in groupByRowChildren)
                {
                    _countGrouped = 0;
                    _countNotGrouped = 0;
                    if (child.RowType == RowType.GroupByRow)
                    {
                        //boucle permettant d'afficher le +/- sur le cote de la feuille Excel pour chaque premier groupe
                        int temp = this.GetQueryElementItemsCount(child as GroupByRow);

                        for (int i = 0; i < temp; i++)
                            _sheet.Rows[_currentMergedRow + i + 1].OutlineLevel = outlinelevel;

                        IterateGroupByRow(child as GroupByRow, outlinelevel + 1);

                        _currentMergedColumn--;
                    }
                    else
                        IterateRow(child);
                }
            }
        }

        /// <summary>
        /// Save a line
        /// </summary>
        /// <param name="row"></param>
        private void IterateRow(Row row)
        {
            int columnIndex = _currentMergedColumn;

            foreach (Cell cell in row.Cells)
            {
                //get the values switch the type of the cell
                if(cell.Column is ComboBoxColumn)
                {
                    string path = ((ComboBoxColumn)(cell.Column)).DisplayMemberPath;
                    string value = (string)cell.Value.GetType().GetProperty(path).GetValue(cell.Value, null);
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = value;
                }
                else if(cell.Column is CheckBoxColumn)
                {
                    var header = cell.Column.HeaderText;
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = Boolean.Parse(cell.Value.ToString()) ? header != null ? header.ToLower() : "Checked" : header != null ? "Not" + header.ToLower() : "Unchecked";
                }
                else if (cell.Column is DateColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((DateColumn) (cell.Column)).SelectedDateFormat;
                }
                else if (cell.Column is HyperlinkColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((HyperlinkColumn)(cell.Column)).Content;
                }
                else if (cell.Column is ImageColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = "Image not supported yet";
                }
                else if (cell.Column is TemplateColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((TemplateColumn)(cell.Column)).ItemTemplate.ToString();
                }
                else if (cell.Column is UnboundColumn)
                {
                    _sheet.Rows[_currentMergedRow + 1].Cells[columnIndex].Value = ((UnboundColumn)(cell.Column)).ItemTemplate.ToString();
                }
                else
                    _sheet.Rows[_currentMergedRow].Cells[columnIndex].Value = cell.Value;

                //Sample to apply a particular style to a row
                    _sheet.Rows[_currentMergedRow].Cells[columnIndex].CellFormat.SetFormatting(_iwshcfValide);

                //affect a width to be sur to see the entire row (could be improve)
                _sheet.Columns[columnIndex].Width = 6000;
                columnIndex++;
            }
            _currentMergedRow++;
        }

        /// <summary>
        /// Method to apply a particular style to a cell and affect a value
        /// </summary>
        /// <param name="cell"></param>
        /// <param name="value"></param>
        private void SetCellValue(WorksheetCell cell, object value)
        {
            cell.Value = value;
            cell.CellFormat.ShrinkToFit = ExcelDefaultableBoolean.True;
            cell.CellFormat.VerticalAlignment = VerticalCellAlignment.Center;
            cell.CellFormat.Alignment = HorizontalCellAlignment.Center;
            cell.CellFormat.Font.Bold = ExcelDefaultableBoolean.True;
        }

    }
}
